#ifndef AIENGINEERROR_H
#define AIENGINEERROR_H

#include <memory>
#include <string>

#include "ai-engine/large-model/aiengine.h"

namespace ai_engine
{
namespace lm
{
// 引擎相关错误分类
enum class EngineErrorCategory {
    INITIALIZATION,
    INPUTERROR,
    OUTPUTERROR
};

#define PLUGINCOMMONERRORCODE   \
    // 通用初始化相关错误       \
    NOMOREMEMORY,               \
                                \
    // 本地模型初始化相关错误   \
    FILENOTFOUND,               \
    VERSIONMISMATCHED,          \
    FILEPERMISSIONDENIED,       \
                                \
    // 云端模型初始化相关错误   \
    NETWORKDISCONNECTED,        \
    NETWORKLATENCYTOOLONG,      \
    FAILEDTOCONNECTSERVER,      \
    SERVERNORESPONSE,           \
    NOSERVERSERVICE,            \
    UNAUTHORIZED,               \
                                \
    // 输入相关错误             \
    TOOLITTLEPARAMS,            \
    TOOMANYPARAMS,              \
    INVALIDPARAMTYPE,           \
    TOOLITTLEDATA,              \
    TOOMANYDATA,                \
    TOOFREQUENTREQUEST,         \
    TOOMANYREQUESTR,

// NLP引擎相关错误码
enum class NlpEngineErrorCode {
    PLUGINCOMMONERRORCODE

    // 输出相关错误
    FAILEDTOANSWER,
    INVALIDWORDS
};

// 语音引擎相关错误码
enum class SpeechEngineErrorCode {
    PLUGINCOMMONERRORCODE

    // 输出相关错误
    UNSUPPORTEDLANGUAGE,
    UNSUPPORTEDSCRIPT,
    INVALIDWAKEUPWORDS
};

// 图像引擎相关错误码
enum class VisionEngineErrorCode {
    PLUGINCOMMONERRORCODE

    // 输出相关错误
    INVALIDPWORDS,
    IMAGEGENERATIONFAILED,
    IMAGECENSORSHIPFAILED
};

class EngineError {
public:
    ~EngineError() = default;

    static std::unique_ptr<EngineError> create(EngineErrorCategory category, NlpEngineErrorCode code, const std::string& message) {
        return std::unique_ptr<EngineError>(new EngineError(AiCapability::Nlp, category, static_cast<int>(code), message));
    }

    static std::unique_ptr<EngineError> create(EngineErrorCategory category, SpeechEngineErrorCode code, const std::string& message) {
        return std::unique_ptr<EngineError>(new EngineError(AiCapability::Speech, category, static_cast<int>(code), message));
    }

    static std::unique_ptr<EngineError> create(EngineErrorCategory category, VisionEngineErrorCode code, const std::string& message) {
        return std::unique_ptr<EngineError>(new EngineError(AiCapability::Vision, category, static_cast<int>(code), message));
    }

    const std::string& getModule() const { return module_; }
    AiCapability getCapability() const { return capability_; }
    EngineErrorCategory getCategory() const { return category_; }
    int getCode() const { return code_; }
    const std::string& getMessage() const { return message_; }

private:
    EngineError(AiCapability capability, EngineErrorCategory category, int code, const std::string& message)
        : capability_(capability), category_(category), code_(code), message_(message) {}

private:
    const std::string module_ = "AI Engine";
    AiCapability capability_;
    EngineErrorCategory category_;
    const int code_;
    const std::string message_;
};
}
}

#endif